@@ -16,23 +16,18 @@ if TYPE_CHECKING:
     from compass.core.util.client import Client
 
     class HierarchyState(TypedDict, total=False):
-        compass: int
+        unit_id: int
         name: Optional[str]
-        Organisation_ID: int
-        Organisation_name: Optional[str]
-        Country_ID: int
-        Country_name: Optional[str]
-        Region_ID: int
-        Region_name: Optional[str]
-        County_ID: int
-        County_name: Optional[str]
-        District_ID: int
-        District_name: Optional[str]
-        Group_ID: int
-        Group_name: Optional[str]
+        # organisation: int  # Always 10000001; mildly redundant
+        country: int
+        region: int
+        county: int
+        district: int
+        group: int
 
 
-TYPE_LEVEL_META = tuple[Union[str, None], Union[scraper.TYPES_ENDPOINT_LEVELS, None], scraper.TYPES_ENDPOINT_LEVELS]
+TYPES_NULLABLE_UNIT_LEVEL = Union[schema.TYPES_UNIT_LEVELS, None]
+TYPE_LEVEL_META = tuple[TYPES_NULLABLE_UNIT_LEVEL, scraper.TYPES_ENDPOINT_LEVELS, scraper.TYPES_ENDPOINT_LEVELS]
 
 
 class Levels(TYPE_LEVEL_META, enum.Enum):
@@ -48,7 +43,7 @@ class Hierarchy:
     def __init__(self, session: Logon):
         """Constructor for Hierarchy."""
         self.client: Client = session._client
-        self.session: Logon = session
+        self.default_hierarchy: schema.HierarchyLevel = session.hierarchy
 
     # See recurseRetrieve in PGS\Needle
     @cache_hooks.cache_result(key=("hierarchy", 1), model_type=schema.UnitData)
@@ -81,8 +76,13 @@ class Hierarchy:
                 When no unit data information has been provided
 
         """
+        if use_default:
+            unit_meta = self.default_hierarchy
+        elif unit_id is not None and level is not None:
+            unit_meta = schema.HierarchyLevel(unit_id=unit_id, level=level)
+        else:
+            raise errors.CompassError("No level data specified! Either `use_default` or `unit_id` and `level` must be set!")
         # Fetch the hierarchy
-        unit_meta = _get_unit_level(self.session, unit_id, level, use_default)
         return schema.UnitData.parse_obj(_get_descendants_level(self.client, unit_meta, recurse_children))
 
     def unique_members(
@@ -118,7 +118,7 @@ class Hierarchy:
         flat_hierarchy = flatten_hierarchy(hierarchy_dict)
 
         # generator for compass unit IDs
-        compass_ids = (unit["compass"] for unit in flat_hierarchy)
+        compass_ids = (unit["unit_id"] for unit in flat_hierarchy)
 
         # get members from the list of IDs
         seen: set[int] = set()  # Unit ID deduplication
@@ -142,49 +142,15 @@ class Hierarchy:
         return scraper.get_members_with_roles_in_unit(self.client, unit_id)
 
 
-def _get_unit_level(
-    session: Logon,
-    unit_id: Optional[int] = None,
-    level: Optional[schema.TYPES_UNIT_LEVELS] = None,
-    use_default: bool = False,
-) -> schema.HierarchyLevel:
-    """Helper function to construct unit level data.
-
-    Unit data can be specified as a pre-constructed model, by passing literals, or
-    by signalling to use the data from the user's current role. If all three
-    options are unset an exception is raised.
-
-    There is a strict priority ordering as follows:
-        1. pre-constructed pydantic model
-        2. literals
-        3. default data
-
-    Returns:
-        Constructed unit level data, as a pydantic model.
-        e.g.:
-            HierarchyLevel(id=..., level="...")
-
-    Raises:
-        CompassError:
-            When no unit data information has been provided
-
-    """
-    if unit_id is not None and level is not None:
-        return schema.HierarchyLevel(unit_id=unit_id, level=level)
-    if use_default:
-        return session.hierarchy  # as this is a property, it will update when roles change
-    raise errors.CompassError("No level data specified! unit_level, id and level, or use_default must be set!")
-
-
 def _get_descendants_level(client: Client, unit_meta: schema.HierarchyLevel, recurse_children: bool) -> dict[str, object]:
     try:
-        level_numeric = Levels[unit_meta.level]
+        level = Levels[unit_meta.level]
     except KeyError:
         valid_levels = [level.name for level in Levels]
         raise errors.CompassError(f"Passed level: {unit_meta.level} is illegal. Valid values are {valid_levels}") from None
     if recurse_children:
-        return _get_descendants_recursive(client, unit_meta.unit_id, level_numeric)
-    return _get_descendants_immediate(client, unit_meta.unit_id, level_numeric)
+        return _get_descendants_recursive(client, unit_meta.unit_id, level)
+    return _get_descendants_immediate(client, unit_meta.unit_id, level)
 
 
 # See recurseRetrieve in PGS\Needle
@@ -241,12 +207,10 @@ def flatten_hierarchy(hierarchy_dict: schema.UnitData) -> Iterator[HierarchyStat
 
 def _flatten(d: Union[schema.UnitData, schema.DescendantData], hierarchy_state: HierarchyState) -> Iterator[HierarchyState]:
     """Generator expresion to recursively flatten hierarchy."""
-    level_name = d.level
     unit_id = d.unit_id
-    name = d.name
-    level_data = hierarchy_state | {f"{level_name}_ID": unit_id, f"{level_name}_name": name}  # type: ignore[operator]
-    yield {"compass": unit_id, "name": name, "section": False} | level_data
+    level_data = hierarchy_state | {d.level.lower(): unit_id}  # type: ignore[operator]
+    yield {"unit_id": unit_id, "name": d.name, "section": False} | level_data
     for child in d.child or []:
         yield from _flatten(child, level_data)
     for section in d.sections:
-        yield {"compass": section.unit_id, "name": section.name, "section": True} | level_data
+        yield {"unit_id": section.unit_id, "name": section.name, "section": True} | level_data
